@@ -76,120 +76,19 @@ class RLModelBase:
             print(f"Error computing transition probabilities: {e}")
             return {} if current_action is not None else pd.DataFrame()
 
-
 @dataclass
-class RLmodel_small(RLModelBase):
-    sma_05: float
-    sma_07: float
-    sma_25: float
-    sma_compare: int
-    is_short: int
-
-    model_file_path: str = f'{Path(__file__).resolve().parent}/small_q_table.npy'
-    state_index_file: str = f'{Path(__file__).resolve().parent}/small_state_to_index.npy'
-    episodes_file: str = None
-
-    def __post_init__(self):
-        self.episodes_file = f'{Path(__file__).resolve().parent}/small_epitrans.npy'
-    def load_qtable(self):
-        with open(self.model_file_path, "rb") as f:
-            q_table = np.load(f)
-        return q_table
-
-    def load_state_index(self):
-        with open(self.state_index_file, "rb") as f:
-            state_to_index = np.load(f, allow_pickle=True).item()
-        return state_to_index
-
-    def load_action_mapping(self):
-        action_mapping = {"go_long": 0, "go_short": 1, "do_nothing": 2}
-        return action_mapping
-
-    def prep_state(self):
-        state = np.array([[self.sma_05, self.sma_07, self.sma_25, self.sma_compare, self.is_short]])
-        if not np.all(np.isfinite(state)):
-            state = np.nan_to_num(state, nan=0.0, posinf=0.0, neginf=0.0)
-        return state
-
-    def predict_action(self):
-        state = self.prep_state()
-        loaded_qtable = self.load_qtable()
-        loaded_state_to_index = self.load_state_index()
-        loaded_mapping = self.load_action_mapping()
-
-
-        state_tuple = tuple(state.flatten())
-        state_index = loaded_state_to_index.get(state_tuple, -1)
-        if not state_index == -1:
-            try:
-                q_values = loaded_qtable[state_index]
-            except ValueError as e:
-                print(e)
-        else:
-            # Create a KDTree from the states in the loaded_state_to_index mapping
-            state_tuples = [tuple(float(x) for x in state) for state in loaded_state_to_index.keys()]
-            kdtree = KDTree(state_tuples)
-
-            # Find the nearest neighbor to the current state
-            distance, index = kdtree.query(state.flatten())
-            nearest_state_tuple = state_tuples[index]
-            new_state_index = loaded_state_to_index[nearest_state_tuple]
-            q_values = loaded_qtable[new_state_index]
-            #raise ValueError("State not found in the state index mapping.")
-        # Compute probabilities using softmax
-
-        def softmax(x):
-            exp_x = np.exp(x - np.max(x))  # Subtract max for numerical stability
-            return exp_x / np.sum(exp_x)
-
-        confidence = softmax(q_values)
-
-        # Map probabilities to action names
-        action_confidence = {
-            action: confidence[index] for action, index in loaded_mapping.items()
-        }
-        best_action_index = np.argmax(q_values)
-
-        action = [action for action, index in loaded_mapping.items() if index == best_action_index][0]
-        # Get transition probabilities for the chosen action
-        trans_proba = self.compute_action_transition_proba(current_action=action)
-        trans_action = max(trans_proba.items(), key=lambda x: x[1])[0] if trans_proba else None
-
-        results_dict = {
-            "raw_state": state,
-            "state_tuple": state_tuple,
-            "best_action_index": best_action_index,
-            "action": action,
-            "confidence": action_confidence,
-            "trans_proba": trans_proba,
-            "trans_action": trans_action
-        }
-        return results_dict
-
-@dataclass
-class RLmodel_large(RLModelBase):
-    opening: float
-    high: float
-    ema_26: float
-    ema_12: float
-    low: float
-    mean_grad_hist: float
-    close: float
-    volume: float
-    sma_25: float
-    long_jcrosk: float
-    short_kdj: int
-    sma_compare: int
+class RLmodel50(RLModelBase):
     ask: float
     bid: float
+    sma_compare: int
     is_short: int
 
-    model_file_path: str = f'{Path(__file__).resolve().parent}/large_q_table.npy'
-    state_index_file: str = f'{Path(__file__).resolve().parent}/large_state_to_index.npy'
+    model_file_path: str = f'{Path(__file__).resolve().parent}/bid50_q_table.npy'
+    state_index_file: str = f'{Path(__file__).resolve().parent}/bid50_state_to_index.npy'
     episodes_file: str = None
 
     def __post_init__(self):
-        self.episodes_file = f'{Path(__file__).resolve().parent}/large_epitrans.npy'
+        self.episodes_file = f'{Path(__file__).resolve().parent}/bid50_epitrans.npy'
 
     def load_qtable(self):
         with open(self.model_file_path, "rb") as f:
@@ -206,11 +105,7 @@ class RLmodel_large(RLModelBase):
         return action_mapping
 
     def prep_state(self):
-        state = np.array([[self.opening, self.high, \
-                          self.ema_26, self.ema_12, self.low, self.mean_grad_hist, \
-                          self.close, self.volume, self.sma_25, self.long_jcrosk, \
-                          self.short_kdj, self.sma_compare, self.ask, self.bid, self.is_short]]
-                    )
+        state = np.array([[self.ask, self.bid, self.sma_compare, self.is_short]])
 
         # Check for NaN or Inf values in the state
         if not np.all(np.isfinite(state)):
@@ -243,8 +138,8 @@ class RLmodel_large(RLModelBase):
             new_state_index = loaded_state_to_index[nearest_state_tuple]
             q_values = loaded_qtable[new_state_index]
             #raise ValueError("State not found in the state index mapping.")
+        
         # Compute probabilities using softmax
-
         def softmax(x):
             exp_x = np.exp(x - np.max(x))  # Subtract max for numerical stability
             return exp_x / np.sum(exp_x)
@@ -255,8 +150,8 @@ class RLmodel_large(RLModelBase):
         action_confidence = {
             action: confidence[index] for action, index in loaded_mapping.items()
         }
+        
         best_action_index = np.argmax(q_values)
-
         action = [action for action, index in loaded_mapping.items() if index == best_action_index][0]
         # Get transition probabilities for the chosen action
         trans_proba = self.compute_action_transition_proba(current_action=action)
@@ -274,18 +169,18 @@ class RLmodel_large(RLModelBase):
         return results_dict
 
 @dataclass
-class RLmodel_bids(RLModelBase):
+class RLmodel74(RLModelBase):
     ask: float
     bid: float
     sma_compare: int
     is_short: int
 
-    model_file_path: str = f'{Path(__file__).resolve().parent}/bids_q_table.pkl'
-    state_index_file: str = f'{Path(__file__).resolve().parent}/bids_state_to_index.pkl'
+    model_file_path: str = f'{Path(__file__).resolve().parent}/bid74_q_table.pkl'
+    state_index_file: str = f'{Path(__file__).resolve().parent}/bid74_state_to_index.pkl'
     episodes_file: str = None
 
     def __post_init__(self):
-        self.episodes_file = f'{Path(__file__).resolve().parent}/bids_epitrans.pkl'
+        self.episodes_file = f'{Path(__file__).resolve().parent}/bid74_epitrans.pkl'
 
     def load_qtable(self):
         with open(self.model_file_path, "rb") as f:
@@ -407,13 +302,142 @@ class RLmodel_bids(RLModelBase):
             "trans_action": trans_action
         }
 
+@dataclass
+class RLmodel86(RLModelBase):
+    ask: float
+    bid: float
+    sma_compare: int
+    is_short: int
 
-def main(model_type: str, sma_05: float = None, sma_07: float = None, 
-         sma_25: float = None, sma_compare: int = None, is_short: int = None,
-         opening: float = None, high: float = None, ema_26: float = None,
-         ema_12: float = None, low: float = None, mean_grad_hist: float = None,
-         close: float = None, volume: float = None, long_jcrosk: float = None,
-         short_kdj: int = None, ask: float = None, bid: float = None):
+    model_file_path: str = f'{Path(__file__).resolve().parent}/bid86_q_table.pkl'
+    state_index_file: str = f'{Path(__file__).resolve().parent}/bid86_state_to_index.pkl'
+    episodes_file: str = None
+
+    def __post_init__(self):
+        self.episodes_file = f'{Path(__file__).resolve().parent}/bid86_epitrans.pkl'
+
+    def load_qtable(self):
+        with open(self.model_file_path, "rb") as f:
+            q_table = list(pickle.load(f))
+        return q_table
+
+    def load_state_index(self):
+        with open(self.state_index_file, "rb") as f:
+            state_to_index = list(pickle.load(f).items())
+        return state_to_index
+
+    def load_action_mapping(self):
+        action_mapping = {"go_long": 0, "go_short": 1, "do_nothing": 2}
+        return action_mapping
+
+    def prep_state(self):
+        state = np.array([[self.ask, self.bid, self.sma_compare, self.is_short]])
+        # Check for NaN or Inf values in the state
+        if not np.all(np.isfinite(state)):
+            state = np.nan_to_num(state, nan=0.0, posinf=0.0, neginf=0.0)
+        return state
+
+    def find_nearest_numeric_state(self, state_tuple, loaded_state_to_index):
+        numeric_states = []
+        numeric_indices = {}
+        
+        for state, idx in loaded_state_to_index:
+            try:
+                # Only process states with the same length as our input state
+                if len(state) == len(state_tuple):
+                    numeric_state = [float(x) for x in state]
+                    numeric_states.append(numeric_state)
+                    numeric_indices[len(numeric_states) - 1] = idx
+            except (ValueError, TypeError):
+                continue
+
+        if not numeric_states:
+            return -1
+
+        numeric_states = np.array(numeric_states)
+        query_state = np.array([float(x) for x in state_tuple])
+        
+        # Use Euclidean distance to find nearest neighbor
+        distances = np.linalg.norm(numeric_states - query_state, axis=1)
+        nearest_idx = np.argmin(distances)
+        
+        return numeric_indices[nearest_idx]
+
+    def predict_action(self):
+        state = self.prep_state()
+        loaded_qtable = self.load_qtable()
+        loaded_state_to_index = self.load_state_index()
+        loaded_mapping = self.load_action_mapping()
+
+        # Convert state to a tuple
+        state_tuple = tuple(state.flatten())
+
+        # Convert loaded_state_to_index to a dictionary for faster lookups
+        state_to_index_dict = dict(loaded_state_to_index)
+
+        # Get the index for the current state
+        state_index = state_to_index_dict.get(state_tuple, -1)
+
+        # If state not found, find nearest numeric state
+        if state_index == -1:
+            state_index = self.find_nearest_numeric_state(state_tuple, loaded_state_to_index)
+            if state_index == -1:
+                # Fallback to default action if no valid state found
+                return {
+                    "raw_state": state,
+                    "state_tuple": state_tuple,
+                    "best_action_index": loaded_mapping["do_nothing"],
+                    "action": "do_nothing",
+                    "confidence": {action: 1.0 if action == "do_nothing" else 0.0 
+                                 for action in loaded_mapping},
+                    "trans_proba": {},
+                    "trans_action": "do_nothing"
+                }
+
+        # Get Q-values for the state
+        q_values = loaded_qtable[state_index]
+        # Normalize q_values to avoid overflow in softmax
+        norm_q_values = q_values + np.random.normal(0, 1e-6, size=q_values.shape)
+
+        # Compute probabilities using softmax
+        def softmax(x):
+            exp_x = np.exp(x - np.max(x))  # Subtract max for numerical stability
+            return exp_x / np.sum(exp_x)
+
+        confidence = softmax(norm_q_values)
+
+        # Map probabilities to action names
+        action_confidence = {
+            action: confidence[index] for action, index in loaded_mapping.items()
+        }
+
+        # Find the best action
+        best_action_index = np.argmax(norm_q_values)
+        action = [action for action, index in loaded_mapping.items() 
+                 if index == best_action_index][0]
+
+        # Find the action with the maximum confidence
+        trans_action = max(action_confidence, key=action_confidence.get)
+
+        # Handle transition probabilities
+        try:
+            trans_proba = self.compute_action_transition_proba(current_action=action)
+        except KeyError:
+            trans_proba = {}
+
+        # Prepare results
+        return {
+            "raw_state": state,
+            "state_tuple": state_tuple,
+            "best_action_index": best_action_index,
+            "action": action,
+            "confidence": action_confidence,
+            "trans_proba": trans_proba,
+            "trans_action": trans_action
+        }
+
+def main(model_type: str, sma_compare: int = None, is_short: int = None,
+         ask: float = None, bid: float = None):
     """
     Main function to handle command line interface.
     
@@ -421,43 +445,29 @@ def main(model_type: str, sma_05: float = None, sma_07: float = None,
         model_type: str - Type of model to use ('small', 'large', or 'bids')
         ... (other parameters specific to each model)
     """
-    if model_type.lower() == "small":
-        if any(x is None for x in [sma_05, sma_07, sma_25, sma_compare, is_short]):
-            raise ValueError("Small model requires: sma_05, sma_07, sma_25, sma_compare, is_short")
-        model = RLmodel_small(
-            sma_05=float(sma_05),
-            sma_07=float(sma_07),
-            sma_25=float(sma_25),
-            sma_compare=int(sma_compare),
-            is_short=int(is_short)
-        )
-    elif model_type.lower() == "large":
-        # Check and create large model
-        if any(x is None for x in [opening, high, ema_26, ema_12, low, mean_grad_hist,
-                                 close, volume, sma_25, long_jcrosk, short_kdj,
-                                 sma_compare, ask, bid, is_short]):
-            raise ValueError("Large model requires all parameters")
-        model = RLmodel_large(
-            opening=float(opening),
-            high=float(high),
-            ema_26=float(ema_26),
-            ema_12=float(ema_12),
-            low=float(low),
-            mean_grad_hist=float(mean_grad_hist),
-            close=float(close),
-            volume=float(volume),
-            sma_25=float(sma_25),
-            long_jcrosk=float(long_jcrosk),
-            short_kdj=int(short_kdj),
-            sma_compare=int(sma_compare),
+    if model_type.lower() == "bid50":
+        if any(x is None for x in [ask, bid, sma_compare, is_short]):
+            raise ValueError("Bid50 model requires: ask, bid, sma_compare, is_short")
+        model = RLmodel50(
             ask=float(ask),
             bid=float(bid),
+            sma_compare=int(sma_compare),
             is_short=int(is_short)
         )
-    elif model_type.lower() == "bids":
+    elif model_type.lower() == "bid74":
+        # Check and create large model
         if any(x is None for x in [ask, bid, sma_compare, is_short]):
-            raise ValueError("Bids model requires: ask, bid, sma_compare, is_short")
-        model = RLmodel_bids(
+            raise ValueError("Bid74 model requires ask, bid, sma_compare, is_short")
+        model = RLmodel74(
+            ask=float(ask),
+            bid=float(bid),
+            sma_compare=int(sma_compare),
+            is_short=int(is_short)
+        )
+    elif model_type.lower() == "bid86":
+        if any(x is None for x in [ask, bid, sma_compare, is_short]):
+            raise ValueError("Bid86 model requires: ask, bid, sma_compare, is_short")
+        model = RLmodel86(
             ask=float(ask),
             bid=float(bid),
             sma_compare=int(sma_compare),
@@ -472,7 +482,7 @@ def main(model_type: str, sma_05: float = None, sma_07: float = None,
 
 # Use case
 #print(RLmodel_large(1, 4, 0, 1, 0, 4, 7, 2, 3, 6, 0, 7, 0, 1, 0).predict_action())
-#print(RLmodel_bids(1, 4, 0, 1).predict_action().get('action'))
+#print(RLmodel86(1, 4, 0, 1).predict_action().get('action'))
 
 
 
