@@ -346,7 +346,7 @@ class RLmodel_bids(RLModelBase):
             self.state_to_index_dict = {}
 
         # Define action mapping
-        self.action_mapping = {"do_nothing": 0, "go_long": 1, "go_short": 2} # Assuming only these two actions for bids
+        self.action_mapping = {"go_long": 0, "go_short": 1} # Assuming only these two actions for bids "do_nothing": 0, 
 
     def _build_kdtree(self):
         """Builds KDTree from numeric states in the state index."""
@@ -626,7 +626,7 @@ def main(model_type: str, sma_05: float = None, sma_07: float = None,
 
 # Use case
 #print(RLmodel_large(1, 4, 0, 1, 0, 4, 7, 2, 3, 6, 0, 7, 0, 1, 0).predict_action())
-#print(RLmodel_bids(1, 4, 0, 1).predict_action())
+#print(RLmodel_bids(0.5,0.512,0,1).predict_action())
 
 
 
